/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.ScriptingBean;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import ca.uvic.csr.shrimp.AttributeConstants;
import ca.uvic.csr.shrimp.FilterConstants;
import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.VisVarConstants;
import ca.uvic.csr.shrimp.AttrToVisVarBean.AttrToVisVarBean;
import ca.uvic.csr.shrimp.DataBean.Artifact;
import ca.uvic.csr.shrimp.DataBean.DataBean;
import ca.uvic.csr.shrimp.DataDisplayBridge.CompositeArcsManager;
import ca.uvic.csr.shrimp.DataDisplayBridge.DataDisplayBridge;
import ca.uvic.csr.shrimp.DataDisplayBridge.RelTypeGroup;
import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.DisplayConstants;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpCompositeArc;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.DisplayBean.ArcStyle.ArcStyle;
import ca.uvic.csr.shrimp.DisplayBean.ArcStyle.CompositeArcStyle;
import ca.uvic.csr.shrimp.FilterBean.FilterBean;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpToolNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.adapter.ZoomModeChangeAdapter;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;
import ca.uvic.csr.shrimp.usercontrols.CloseSelectedNodesAdapter;
import ca.uvic.csr.shrimp.usercontrols.CollapseAllDescendentsAdapter;
import ca.uvic.csr.shrimp.usercontrols.FilterSelectedArtifactsAdapter;
import ca.uvic.csr.shrimp.usercontrols.FocusOnHomeAdapter;
import ca.uvic.csr.shrimp.usercontrols.GroupSelectedArtifactsAdapter;
import ca.uvic.csr.shrimp.usercontrols.LayoutModeChangeAdapter;
import ca.uvic.csr.shrimp.usercontrols.OpenAllAdapter;
import ca.uvic.csr.shrimp.usercontrols.OpenSelectedNodesAdapter;
import ca.uvic.csr.shrimp.usercontrols.RedoActionAdapter;
import ca.uvic.csr.shrimp.usercontrols.RenameSelectedArtifactsAdapter;
import ca.uvic.csr.shrimp.usercontrols.SelectAllChildrenAdapter;
import ca.uvic.csr.shrimp.usercontrols.SelectAllDescendantsAdapter;
import ca.uvic.csr.shrimp.usercontrols.SelectConnectedNodesAdapter;
import ca.uvic.csr.shrimp.usercontrols.SelectInverseSiblingsAdapter;
import ca.uvic.csr.shrimp.usercontrols.SnapShotAdapter;
import ca.uvic.csr.shrimp.usercontrols.UndoActionAdapter;
import ca.uvic.csr.shrimp.usercontrols.UnfilterAllByIdAdapter;
import ca.uvic.csr.shrimp.usercontrols.UngroupSelectedArtifactsAdapter;

/**
 * @author Nasir Rather
 *	
 *	@see ca.uvic.csr.shrimp.ScriptingBean.ScriptingConstants
 */
public class DefaultMainViewScriptingBean implements MainViewScriptingBean {

	private SnapShotAdapter snapShotAdapter;

	private LayoutModeChangeAdapter layoutChildrenAdapter;
	private LayoutModeChangeAdapter layoutSelectedAdapter;

	private OpenSelectedNodesAdapter openSelectedNodesAdapter;
	private CloseSelectedNodesAdapter closeSelectedNodesAdapter;
	private CollapseAllDescendentsAdapter collapseAllDescendentsAdapter;

	private FilterSelectedArtifactsAdapter filterSelectedArtifactsAdapter;
	// @tag Shrimp(grouping)
	private GroupSelectedArtifactsAdapter groupSelectedArtifactsAdapter;
	private RenameSelectedArtifactsAdapter renameSelectedNodeAdapter;
	private UngroupSelectedArtifactsAdapter ungroupSelectedArtifactsAdapter;
	
	private UnfilterAllByIdAdapter unfilterAllByIdAdapter;

	private FocusOnHomeAdapter focusOnHomeAdapter;

	private OpenAllAdapter openAllAdapter;

	private UndoActionAdapter undoActionAdapter;
	private RedoActionAdapter redoActionAdapter;

	private SelectAllChildrenAdapter selectAllChildrenAdapter;
	private SelectAllDescendantsAdapter selectAllDescendentsAdapter;
	private SelectInverseSiblingsAdapter selectInverseSiblingsAdapter;
	private SelectConnectedNodesAdapter selectConnectedNodesAdapter;

	private ZoomModeChangeAdapter zoomModeChangeAdapter;
	
	private ShrimpProject project;

	
	public DefaultMainViewScriptingBean(ShrimpProject project) {
		this.project = project;
	}

	/**
	 * @param zoomModeChangeAdapter
	 */
	public void setZoomModeChangeAdapter(ZoomModeChangeAdapter zoomModeChangeAdapter) {
		this.zoomModeChangeAdapter = zoomModeChangeAdapter;
	}

	/**
	 * @param unfilterAllByIdAdapter
	 */
	public void setUnfilterAllByIdAdapter(UnfilterAllByIdAdapter unfilterAllByIdAdapter) {
		this.unfilterAllByIdAdapter = unfilterAllByIdAdapter;
	}

	/**
	 * @param layoutChildrenAdapter
	 */
	public void setLayoutChildrenAdapter(LayoutModeChangeAdapter layoutChildrenAdapter) {
		this.layoutChildrenAdapter = layoutChildrenAdapter;
	}

	/**
	 * @param layoutSelectedAdapter
	 */
	public void setLayoutSelectedAdapter(LayoutModeChangeAdapter layoutSelectedAdapter) {
		this.layoutSelectedAdapter = layoutSelectedAdapter;
	}

	/**
	 * @param closeSelectedNodesAdapter
	 */
	public void setCloseSelectedNodesAdapter(CloseSelectedNodesAdapter closeSelectedNodesAdapter) {
		this.closeSelectedNodesAdapter = closeSelectedNodesAdapter;
	}

	/**
	 * @param collapseAllDescendentsAdapter
	 */
	public void setCollapseAllDescendentsAdapter(CollapseAllDescendentsAdapter collapseAllDescendentsAdapter) {
		this.collapseAllDescendentsAdapter = collapseAllDescendentsAdapter;
	}

	/**
	 * @param filterSelectedArtifactsAdapter
	 */
	public void setFilterSelectedArtifactsAdapter(FilterSelectedArtifactsAdapter filterSelectedArtifactsAdapter) {
		this.filterSelectedArtifactsAdapter = filterSelectedArtifactsAdapter;
	}

	/**
	 * @param focusOnHomeAdapter
	 */
	public void setFocusOnHomeAdapter(FocusOnHomeAdapter focusOnHomeAdapter) {
		this.focusOnHomeAdapter = focusOnHomeAdapter;
	}

	/**
	 * @param openAllAdapter
	 */
	public void setOpenAllAdapter(OpenAllAdapter openAllAdapter) {
		this.openAllAdapter = openAllAdapter;
	}

	/**
	 * @param openSelectedNodesAdapter
	 */
	public void setOpenSelectedNodesAdapter(OpenSelectedNodesAdapter openSelectedNodesAdapter) {
		this.openSelectedNodesAdapter = openSelectedNodesAdapter;
	}

	/**
	 * @param redoActionAdapter
	 */
	public void setRedoActionAdapter(RedoActionAdapter redoActionAdapter) {
		this.redoActionAdapter = redoActionAdapter;
	}

	/**
	 * @param selectAllChildrenAdapter
	 */
	public void setSelectAllChildrenAdapter(SelectAllChildrenAdapter selectAllChildrenAdapter) {
		this.selectAllChildrenAdapter = selectAllChildrenAdapter;
	}

	/**
	 * @param selectAllDescendentsAdapter
	 */
	public void setSelectAllDescendentsAdapter(SelectAllDescendantsAdapter selectAllDescendentsAdapter) {
		this.selectAllDescendentsAdapter = selectAllDescendentsAdapter;
	}

	/**
	 * @param snapShotAdapter
	 */
	public void setSnapShotAdapter(SnapShotAdapter snapShotAdapter) {
		this.snapShotAdapter = snapShotAdapter;
	}

	/**
	 * @param undoActionAdapter
	 */
	public void setUndoActionAdapter(UndoActionAdapter undoActionAdapter) {
		this.undoActionAdapter = undoActionAdapter;
	}

	/**
	 * @see MainViewScriptingBean#setHierarchy(String[])
	 */
	public void setHierarchy(String [] cprels) {
		try {
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			shrimpView.setCprels(cprels, false, true);
			shrimpView.addDefaultRootNodes(true);
		} catch (ShrimpToolNotFoundException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#layoutChildren(java.lang.String)
	 */
	public void layoutChildren(String newLayoutMode) {
		if (layoutChildrenAdapter != null) {
			layoutChildrenAdapter.changeLayout(newLayoutMode, false);
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#layoutSelected(java.lang.String)
	 */
	public void layoutSelected(String newLayoutMode) {
		if (layoutSelectedAdapter != null) {
			layoutSelectedAdapter.changeLayout(newLayoutMode, false);
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#openSelectedNodes()
	 */
	public void openSelectedNodes() {
		if (openSelectedNodesAdapter != null)
			openSelectedNodesAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#closeAllDescendents()
	 */
	public void closeAllDescendents() {
		if (collapseAllDescendentsAdapter != null)
			collapseAllDescendentsAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#filterSelectedNodes()
	 */
	public void filterSelectedNodes() {
		if (filterSelectedArtifactsAdapter != null)
			filterSelectedArtifactsAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#groupSelectedNodes()
	 * 
	 */
	public void groupSelectedNodes() {
		if (groupSelectedArtifactsAdapter != null)
			groupSelectedArtifactsAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#ungroupSelectedNodes()
	 * 
	 */
	public void ungroupSelectedNodes() {
		if (ungroupSelectedArtifactsAdapter != null)
			ungroupSelectedArtifactsAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#ungroupSelectedNodes()
	 * 
	 */
	public void renameSelectedNode() {
		if (renameSelectedNodeAdapter != null)
			renameSelectedNodeAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#focusOnHome()
	 */
	public void focusOnHome() {
		if (focusOnHomeAdapter != null)
			focusOnHomeAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#openAllDescendents()
	 */
	public void openAllDescendents() {
		if (openAllAdapter != null)
			openAllAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#closeSelectedNodes()
	 */
	public void closeSelectedNodes() {
		if (closeSelectedNodesAdapter != null)
			closeSelectedNodesAdapter.closeSelectedNodes();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#redoAction()
	 */
	public void redoAction() {
		if (redoActionAdapter != null)
			redoActionAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#selectAllChildren()
	 */
	public void selectAllChildren() {
		if (selectAllChildrenAdapter != null)
			selectAllChildrenAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#selectAllDescendents()
	 */
	public void selectAllDescendents() {
		if (selectAllDescendentsAdapter != null)
			selectAllDescendentsAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#takeSnapShot(boolean)
	 */
	public void takeSnapShot(boolean askForComments) {
		if (snapShotAdapter != null)
			snapShotAdapter.takeSnapShot(askForComments);
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#unFilterAllArtifacts()
	 */
	public void unFilterAllArtifacts() {
		if (unfilterAllByIdAdapter != null)
			unfilterAllByIdAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#undoAction()
	 */
	public void undoAction() {
		if (undoActionAdapter != null)
			undoActionAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#changeZoomMode(java.lang.String)
	 */
	public void changeZoomMode(String newMode) {
		if (zoomModeChangeAdapter != null)
			zoomModeChangeAdapter.changeZoomMode(newMode);
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#filterAllArcs(boolean)
	 */
	public void filterAllArcs(boolean filter) {
		if (project != null) {
			try {
				ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
				FilterBean filterBean = (FilterBean) shrimpView.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);
				DataBean dataBean = (DataBean) project.getBean(ShrimpProject.DATA_BEAN);

				Vector types = dataBean.getRelationshipTypes(true, true);
				filterBean.addRemoveNominalAttrValues(AttributeConstants.NOM_ATTR_REL_TYPE, String.class, FilterConstants.RELATIONSHIP_FILTER_TYPE, types, filter);

			} catch (ShrimpToolNotFoundException e) {
				e.printStackTrace();
			} catch (BeanNotFoundException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#filterAllNodes(boolean)
	 */
	public void filterAllNodes(boolean filter) {
		if (project != null) {
			try {
				ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
				FilterBean filterBean = (FilterBean) shrimpView.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);
				DataBean dataBean = (DataBean) project.getBean(ShrimpProject.DATA_BEAN);

				Vector types = dataBean.getArtifactTypes(true, true);
				filterBean.addRemoveNominalAttrValues(AttributeConstants.NOM_ATTR_ARTIFACT_TYPE, String.class, FilterConstants.ARTIFACT_FILTER_TYPE, types, filter);

			} catch (ShrimpToolNotFoundException e) {
				e.printStackTrace();
			} catch (BeanNotFoundException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#filterArcsByType(java.lang.String, boolean)
	 */
	public void filterArcsByType(String type, boolean filter) {
		if (project != null) {
			try {
				ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
				FilterBean filterBean = (FilterBean) shrimpView.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);
				filterBean.addRemoveSingleNominalAttrValue(AttributeConstants.NOM_ATTR_REL_TYPE, String.class, FilterConstants.RELATIONSHIP_FILTER_TYPE, type, filter);
			} catch (ShrimpToolNotFoundException e) {
				e.printStackTrace();
			} catch (BeanNotFoundException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#filterNodesByType(java.lang.String, boolean)
	 */
	public void filterNodesByType(String type, boolean filter) {
		if (project != null) {
			try {
				ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
				FilterBean filterBean = (FilterBean) shrimpView.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);
				filterBean.addRemoveSingleNominalAttrValue(AttributeConstants.NOM_ATTR_ARTIFACT_TYPE, String.class, FilterConstants.ARTIFACT_FILTER_TYPE, type, filter);
			} catch (ShrimpToolNotFoundException e) {
				e.printStackTrace();
			} catch (BeanNotFoundException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#filterArtifactDataType(java.lang.String, boolean)
	 */
	public void filterArtifactDataType(String type, boolean filter) {
        if (project != null) {
            try {
                FilterBean dataFilterBean = (FilterBean) project.getBean(ShrimpProject.DATA_FILTER_BEAN);
                dataFilterBean.addRemoveSingleNominalAttrValue(AttributeConstants.NOM_ATTR_ARTIFACT_TYPE, String.class, FilterConstants.ARTIFACT_FILTER_TYPE, type, filter);
            } catch (BeanNotFoundException e) {
                e.printStackTrace();
            }
        }
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#filterRelationshipDataType(java.lang.String, boolean)
	 */
	public void filterRelationshipDataType(String type, boolean filter) {
        if (project != null) {
            try {
                FilterBean dataFilterBean = (FilterBean) project.getBean(ShrimpProject.DATA_FILTER_BEAN);
                dataFilterBean.addRemoveSingleNominalAttrValue(AttributeConstants.NOM_ATTR_REL_TYPE, String.class, FilterConstants.RELATIONSHIP_FILTER_TYPE, type, filter);
            } catch (BeanNotFoundException e) {
                e.printStackTrace();
            }
        }
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#filterAllArtifactDataTypes(boolean)
	 */
	public void filterAllArtifactDataTypes(boolean filter) {
		if (project != null) {
			try {
				DataBean dataBean = (DataBean) project.getBean(ShrimpProject.DATA_BEAN);
				Vector types = dataBean.getArtifactTypes(!filter, true);
                FilterBean dataFilterBean = (FilterBean) project.getBean(ShrimpProject.DATA_FILTER_BEAN);
                dataFilterBean.addRemoveNominalAttrValues(AttributeConstants.NOM_ATTR_ARTIFACT_TYPE, String.class, FilterConstants.ARTIFACT_FILTER_TYPE, types, filter);
			} catch (BeanNotFoundException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#filterAllRelationshipDataTypes(boolean)
	 */
	public void filterAllRelationshipDataTypes(boolean filter) {
		if (project != null) {
			try {
				DataBean dataBean = (DataBean) project.getBean(ShrimpProject.DATA_BEAN);
				Vector types = dataBean.getRelationshipTypes(!filter, true);
                FilterBean dataFilterBean = (FilterBean) project.getBean(ShrimpProject.DATA_FILTER_BEAN);
                dataFilterBean.addRemoveNominalAttrValues(AttributeConstants.NOM_ATTR_REL_TYPE, String.class, FilterConstants.RELATIONSHIP_FILTER_TYPE, types, filter);
			} catch (BeanNotFoundException e) {
				e.printStackTrace();
			}
		}
	}


	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#refresh()
	 */
	public void refresh() {
		if (project != null) {
			project.refresh();
		}
	}


	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#selectNodesByName(String, boolean, boolean, boolean, boolean)
	 */
	public void selectNodesByName(String nameToFind, boolean selectAllOccurances, boolean clearPreviouslySelected, boolean exactMatch, boolean caseSensitive) {
		try {
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			final SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);

			final Vector selectedNodes = new Vector();
			if (!clearPreviouslySelected) {
				Vector previouslySelected = (Vector) selectorBean.getSelected(SelectorBeanConstants.SELECTED_NODES);
				if (previouslySelected != null)
					selectedNodes.addAll(previouslySelected);
			}

			final Vector nodes = new Vector();
			Vector allNodes = displayBean.getAllNodes();
			for (Iterator iterator = allNodes.iterator(); iterator.hasNext();) {
				ShrimpNode node = (ShrimpNode) iterator.next();
				String tmpName = node.getName();
				boolean match = false;
				if (caseSensitive) {
				    if (exactMatch) {
				        match = tmpName.equals(nameToFind);
				    } else {
				        match = tmpName.indexOf(nameToFind) != -1;
				    }
				} else {
				    if (exactMatch) {
				        match = tmpName.toLowerCase().equals(nameToFind.toLowerCase());
				    } else {
				        match = tmpName.toLowerCase().indexOf(nameToFind.toLowerCase()) != -1;
				    }
				}
				
				if (match) {
					nodes.add(node);
					if (!selectAllOccurances) {
						break;
					}
				}
			}
			selectedNodes.addAll(nodes);
			
			selectorBean.setSelected(SelectorBeanConstants.SELECTED_NODES, selectedNodes);
			if (!nodes.isEmpty()) {
				selectorBean.setSelected(SelectorBeanConstants.TARGET_OBJECT, nodes.firstElement());
			}
		} catch (ShrimpToolNotFoundException stnfe) {
			stnfe.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#navigateToNode(java.lang.String)
	 */
	public void navigateToNode(String name) {
		try {
			DataBean dataBean = (DataBean) project.getBean(ShrimpProject.DATA_BEAN);
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);

			ShrimpNode validNode = null;
			// look through all created nodes for a node with the given name
			for (Iterator iterator = displayBean.getAllNodes().iterator(); iterator.hasNext() && validNode == null;) {
				ShrimpNode tempNode = (ShrimpNode) iterator.next();
				if (tempNode.getName().equals(name)) {
					validNode = tempNode;
				}
			}
			if (validNode == null) {
				//pull out all the stops and look through ALL the data for an artifact with this name
				//this could take a while if have huge data store
				Artifact artifact = dataBean.findFirstArtifact(AttributeConstants.NOM_ATTR_ARTIFACT_NAME, name, displayBean.getCprels());
				if (artifact != null) {
					Vector nodes = displayBean.getDataDisplayBridge().getShrimpNodes(artifact, true);
					if (nodes.size()>0) {
						validNode = (ShrimpNode) nodes.firstElement();
					}
				}
			}
			if (validNode != null) {
				shrimpView.navigateToObject(validNode);
			}
		} catch (ShrimpToolNotFoundException stnfe) {
			stnfe.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#pause(long)
	 */
	public void pause(long time) {
		try {
			//wait(time);
			Thread.sleep(time);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#alert(java.lang.String)
	 */
	public void alert(String message) {
		alert(message, "Scripting message");
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#prompt(java.lang.String, java.lang.String)
	 */
	public String prompt(String message, String title) {
		String input = "";
		try {
			ShrimpView shrimpView =	(ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
			boolean displayEnabled = displayBean.isEnabled();
			displayBean.setEnabled(false);
			try {
				input = JOptionPane.showInputDialog(ApplicationAccessor.getApplication().getParentFrame(),	message, title,	JOptionPane.PLAIN_MESSAGE);
			} catch (Exception e) {
				displayBean.setEnabled(displayEnabled);
			}
		} catch (ShrimpToolNotFoundException e) {
			e.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		} 
		return input;
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#alert(java.lang.String, java.lang.String)
	 */
	public void alert(String message, String title) {
		JOptionPane.showMessageDialog(ApplicationAccessor.getApplication().getParentFrame(), message, title, JOptionPane.PLAIN_MESSAGE);
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#showDialog(java.lang.String)
	 */
	public void showDialog(String message) {
		showDialog(message, "Scripting message");
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#showDialog(java.lang.String, java.lang.String)
	 */
	public void showDialog(String message, String title) {
		//Create the dialog.
		Frame frame = ApplicationAccessor.getApplication().getParentFrame();
		final JDialog dialog = new JDialog(frame, title);

		//Add contents to it. It must have a close button,
		//since some L&Fs (notably Java/Metal) don't provide one
		//in the window decorations for dialogs.
		//JLabel label =	new JLabel(message);
		JTextArea textArea = new JTextArea(message);
		textArea.setSize(new Dimension(100, 300));
		textArea.setLineWrap(true);
		textArea.setWrapStyleWord(true);
		//textArea.setHorizontalAlignment(JLabel.CENTER);
		textArea.setFont(textArea.getFont().deriveFont(Font.PLAIN, 14.0f));

		JButton closeButton = new JButton("Close");
		closeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialog.setVisible(false);
				dialog.dispose();
			}
		});
		JPanel closePanel = new JPanel();
		closePanel.setLayout(new BoxLayout(closePanel, BoxLayout.LINE_AXIS));
		closePanel.add(Box.createHorizontalGlue());
		closePanel.add(closeButton);
		closePanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 5));

		JPanel contentPane = new JPanel(new BorderLayout());
		contentPane.add(textArea, BorderLayout.CENTER);
		contentPane.add(closePanel, BorderLayout.PAGE_END);
		contentPane.setOpaque(true);
		dialog.setContentPane(contentPane);

		//Show it.
		dialog.setSize(new Dimension(300, 150));
		dialog.setLocationRelativeTo(frame);
		dialog.setVisible(true);
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#setLabelMode(java.lang.String)
	 */
	public void setLabelMode(String mode) {
		try {
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
			SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
			displayBean.setDefaultLabelMode(mode);
			selectorBean.setSelected(DisplayConstants.LABEL_MODE, mode);
		} catch (ShrimpToolNotFoundException e) {
			e.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#selectNodesByType(java.lang.String, boolean)
	 */
	public void selectNodesByType(String type, boolean clearPreviouslySelected) {
		try {
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
			SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
			Vector newSelected = clearPreviouslySelected ? new Vector () : (Vector) selectorBean.getSelected(SelectorBeanConstants.SELECTED_NODES);
			Vector visibleNodes = displayBean.getVisibleNodes();
			for (Iterator iter = visibleNodes.iterator(); iter.hasNext();) {
				ShrimpNode tmpNode = (ShrimpNode) iter.next();
				if (tmpNode.getArtifact().getType().equals(type) && !newSelected.contains(tmpNode)) {
					newSelected.add(tmpNode);
				}
			}
			selectorBean.setSelected(SelectorBeanConstants.SELECTED_NODES, newSelected);				
		} catch (ShrimpToolNotFoundException e) {
			e.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#selectInverseSiblings()
	 */
	public void selectInverseSiblings() {
		selectInverseSiblingsAdapter.startAction();
	}

	/**
	 * @param adapter
	 */
	public void setSelectInverseSiblingsAdapter(SelectInverseSiblingsAdapter adapter) {
		selectInverseSiblingsAdapter = adapter;
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#selectConnectedNodes()
	 */
	public void selectConnectedNodes() {
		selectConnectedNodesAdapter.startAction();
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#createComposites(java.lang.String[])
	 */
	public void createComposites(String[] arcTypes) {
		try {
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
			AttrToVisVarBean attrToVisVarBean = (AttrToVisVarBean) project.getBean(ShrimpProject.ATTR_TO_VIS_VAR_BEAN);
			DataDisplayBridge dataDisplayBridge = displayBean.getDataDisplayBridge();
			CompositeArcsManager compositeArcsManager = dataDisplayBridge.getCompositeArcsManager();
			compositeArcsManager.clear();
			String groupName = "";
			for (int i = 0; i < arcTypes.length; i++) {
				String arcType = arcTypes[i];
				groupName += (i > 0) ? ", " + arcType : arcType;
			}
			groupName += " Group";
			RelTypeGroup group = compositeArcsManager.createRelTypeGroup(groupName);
			for (int i = 0; i < arcTypes.length; i++) {
				String arcType = arcTypes[i];
				compositeArcsManager.createRelTypeGroup(arcType);
			}
			Color groupColor = Color.RED;
			if (arcTypes.length == 1) {
				Color typeColor = (Color)attrToVisVarBean.getVisVarValue(AttributeConstants.NOM_ATTR_REL_TYPE, VisVarConstants.VIS_VAR_ARC_COLOR,arcTypes[0]); 
				if (typeColor != null) {
					groupColor = typeColor; 
				}
			}
			ArcStyle arcStyle = new CompositeArcStyle ();
			group.setCompositeColor(groupColor);
			compositeArcsManager.setCompositesEnabled(group, true);

			attrToVisVarBean.setNominalVisualVariableValue(AttributeConstants.NOM_ATTR_REL_TYPE, VisVarConstants.VIS_VAR_ARC_COLOR, group.getGroupName(), groupColor);
			attrToVisVarBean.setNominalVisualVariableValue(AttributeConstants.NOM_ATTR_REL_TYPE, VisVarConstants.VIS_VAR_ARC_STYLE, group.getGroupName(), arcStyle);

			int threshold = 0;
			Collection arcs = compositeArcsManager.getCompositeArcs();
			for (Iterator iterator = arcs.iterator(); iterator.hasNext();) {
				ShrimpCompositeArc arc = (ShrimpCompositeArc) iterator.next();
				if (arc.isAtHighLevel()) {
					if (arc.getArcCount() < threshold) {
						displayBean.setVisible(arc, false, false);
					} else {
						displayBean.setVisible(arc, true, false);
					}
				}
			}
		} catch (ShrimpToolNotFoundException e) {
			e.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#clearComposites()
	 */
	public void clearComposites() {
		try {
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
			DataDisplayBridge dataDisplayBridge = displayBean.getDataDisplayBridge();
			CompositeArcsManager compositeArcsManager = dataDisplayBridge.getCompositeArcsManager();
			compositeArcsManager.clear();
		} catch (ShrimpToolNotFoundException e) {
			e.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}

    public void setSelectConnectedNodesAdapter(SelectConnectedNodesAdapter selectConnectedNodesAdapter) {
        this.selectConnectedNodesAdapter = selectConnectedNodesAdapter;
    }
    
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.ScriptingBean.MainViewScriptingBean#selectRoots()
     */
    public void selectRoots() {
		try {
			ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
			SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
			Vector newSelected = new Vector (displayBean.getDataDisplayBridge().getRootNodes());
			selectorBean.setSelected(SelectorBeanConstants.SELECTED_NODES, newSelected);				
		} catch (ShrimpToolNotFoundException e) {
			e.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
    }

	// @tag Shrimp(grouping)
	public void setGroupSelectedArtifactsAdapter(
			GroupSelectedArtifactsAdapter groupSelectedArtifactsAdapter) {
		this.groupSelectedArtifactsAdapter = groupSelectedArtifactsAdapter;
	}

	// @tag Shrimp(grouping)
	public void setUngroupSelectedArtifactsAdapter(
			UngroupSelectedArtifactsAdapter ungroupSelectedArtifactsAdapter) {
		this.ungroupSelectedArtifactsAdapter = ungroupSelectedArtifactsAdapter;
	}

	// @tag Shrimp(grouping)
	public void setRenameSelectedNodeAdapter(
			RenameSelectedArtifactsAdapter renameSelectedNodeAdapter) {
		this.renameSelectedNodeAdapter = renameSelectedNodeAdapter;
	}
}